package stereoView;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.media.j3d.Canvas3D;
import javax.media.j3d.Node;
import javax.media.j3d.Transform3D;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.vecmath.Point3d;

import com.sun.j3d.utils.universe.ViewInfo;

public class StereoPreferencesPanel extends JPanel implements ChangeListener, ActionListener{

	private static final long serialVersionUID = -2237510667725947315L;

	Canvas3D canvas = null;

	JSlider slider = null;
	JLabel sliderpos = null;
	JButton button = null;
	JTextField text = null;
	
	EyeFocus ef = null;

	private Node focus_node;
	
	static private int MIN = 0;
	static private int MAX = 250;
	static private int DEFAULT = 150;
	static private double SCALE = 1000;
	
	static private int max_dist = 15;
	
	
	private double getValue() {
		double value;
		value = ((double) slider.getValue()) / SCALE;
		return value;
	}
	
	private String getValueString() {
		return String.format("%.3f", getValue());
	}
	
	public StereoPreferencesPanel(Canvas3D canvas){
		this.canvas = canvas;
		this.ef = new EyeFocus(canvas);
//		((Canvas)canvas).setPrefs(this);
		slider = new JSlider(MIN, MAX, DEFAULT);
		slider.setPaintTicks(true);
		sliderpos = new JLabel();
		sliderpos.setText(getValueString());
		slider.addChangeListener(this);
		button = new JButton("apply");
		button.addActionListener(this);
		text = new JTextField(4);
		this.add(slider);
		this.add(sliderpos);
//		this.add(text);
//		this.add(button);
	}

	public void stateChanged(ChangeEvent e) {
		sliderpos.setText(getValueString());
		setEyeDistance(getValue());
	}
	
	public void setEyeDistance(double d) {
			double lx = -d/2;
			Point3d left = new Point3d();
			canvas.getView().getPhysicalBody().getLeftEyePosition(left);
			left.setX(lx);
			
			double rx = d/2;
			Point3d right= new Point3d();
			canvas.getView().getPhysicalBody().getRightEyePosition(right);
			right.setX(rx);
			
			canvas.setLeftManualEyeInImagePlate(left);
			canvas.setRightManualEyeInImagePlate(right);
			
			canvas.getView().getPhysicalBody().setLeftEyePosition(left);
			canvas.getView().getPhysicalBody().setRightEyePosition(right);
			
	}
	
	public void updateEyes() {
		
//		if (!canvas.getStereoEnable())
//			return;
		if (focus_node == null)
			return;
		
		Point3d eye_l = new Point3d();
		Point3d eye_r = new Point3d();
		Transform3D el = new Transform3D();
		Transform3D er = new Transform3D();
		
		Transform3D obj_t = new Transform3D();
		
		Transform3D r = new Transform3D();
		Transform3D l = new Transform3D();
		Point3d ro = new Point3d();
		Point3d lo = new Point3d();
		
		ViewInfo v = new ViewInfo(canvas.getView());
		
		v.getEyeToVworld(canvas, el, er);

		el.transform(eye_l);
		er.transform(eye_r);
		
//		System.out.println("Left: " + eye_l);
//		System.out.println("Right: " + eye_r);
		
		focus_node.getLocalToVworld(obj_t);
		
		r.set(obj_t);
		l.set(obj_t);
		
		r.mul(er);
		l.mul(el);
		
		r.transform(ro);
		l.transform(lo);
		
//		System.out.println("Dist_Os: " + lo.distance(ro));
//		System.out.println("Dist_LE_LO:" + eye_l.distance(lo));
//		System.out.println("Dist_RE_RO:" + eye_r.distance(ro));
	 	
		double dist = eye_l.distance(lo) + 1;
		
		double eye_dist = getValue() - (getValue()/max_dist) * dist;
		if (dist < max_dist)
			setEyeDistance(eye_dist);
		else
			setEyeDistance(0);
//		System.out.println(eye_dist);
	}

	public void setFocusObject(Node focus_obj){
		ef.setFocusObject(focus_obj);
		this.focus_node = focus_obj;
	}

	public void actionPerformed(ActionEvent e) {
		updateEyes();
	}
	
}